@@ -37,6 +37,9 @@
 #include <snapper/SystemCmd.h>
 #include <snapper/SnapperDefines.h>
 #include <snapper/XAttributes.h>
+#ifdef ENABLE_ROLLBACK
+#include <snapper/Filesystem.h>
+#endif
 
 #include "utils/text.h"
 #include "utils/Table.h"
@@ -44,6 +47,8 @@
 
 #include "commands.h"
 #include "cleanup.h"
+#include "errors.h"
+#include "misc.h"
 
 
 using namespace snapper;
@@ -78,136 +83,12 @@ string config_name = "root";
 bool no_dbus = false;
 
 
-Snapshots::iterator
-read_num(Snapper* snapper, const string& str)
-{
-    Snapshots& snapshots = snapper->getSnapshots();
-
-    istringstream s(str);
-    unsigned int num = 0;
-    s >> num;
-
-    if (s.fail() || !s.eof())
-    {
-        cerr << sformat(_("Invalid snapshot '%s'."), str.c_str()) << endl;
-        exit(EXIT_FAILURE);
-    }
-
-    Snapshots::iterator snap = snapshots.find(num);
-    if (snap == snapshots.end())
-    {
-        cerr << sformat(_("Snapshot '%u' not found."), num) << endl;
-        exit(EXIT_FAILURE);
-    }
-
-    return snap;
-}
-
-
-unsigned int
-read_num(const string& str)
-{
-    istringstream s(str);
-    unsigned int num = 0;
-    s >> num;
-
-    if (s.fail() || !s.eof())
-    {
-	cerr << sformat(_("Invalid snapshot '%s'."), str.c_str()) << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    return num;
-}
-
-
-pair<unsigned int, unsigned int>
-read_nums(const string& str, const string& delim = "..")
-{
-    string::size_type pos = str.find(delim);
-    if (pos == string::npos)
-    {
-	cerr << _("Invalid snapshots.") << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    unsigned int num1 = read_num(str.substr(0, pos));
-    unsigned int num2 = read_num(str.substr(pos + delim.size()));
-
-    if (num1 == num2)
-    {
-	cerr << _("Identical snapshots.") << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    return pair<unsigned int, unsigned int>(num1, num2);
-}
-
-
-map<string, string>
-read_userdata(const string& s, const map<string, string>& old = map<string, string>())
-{
-    map<string, string> userdata = old;
-
-    list<string> tmp;
-    boost::split(tmp, s, boost::is_any_of(","), boost::token_compress_on);
-    if (tmp.empty())
-    {
-	cerr << _("Invalid userdata.") << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    for (list<string>::const_iterator it = tmp.begin(); it != tmp.end(); ++it)
-    {
-	string::size_type pos = it->find("=");
-	if (pos == string::npos)
-	{
-	    cerr << _("Invalid userdata.") << endl;
-	    exit(EXIT_FAILURE);
-	}
-
-	string key = boost::trim_copy(it->substr(0, pos));
-	string value = boost::trim_copy(it->substr(pos + 1));
-
-	if (key.empty())
-	{
-	    cerr << _("Invalid userdata.") << endl;
-	    exit(EXIT_FAILURE);
-	}
-
-	if (value.empty())
-	    userdata.erase(key);
-	else
-	    userdata[key] = value;
-    }
-
-    return userdata;
-}
-
-
-string
-show_userdata(const map<string, string>& userdata)
-{
-    string s;
-
-    for (map<string, string>::const_iterator it = userdata.begin(); it != userdata.end(); ++it)
-    {
-	if (!s.empty())
-	    s += ", ";
-	s += it->first + "=" + it->second;
-    }
-
-    return s;
-}
-
-
 struct MyFiles : public Files
 {
     friend class MyComparison;
 
     MyFiles(const FilePaths* file_paths)
 	: Files(file_paths) {}
-
 };
 
 
@@ -241,7 +122,6 @@ struct MyComparison
     FilePaths file_paths;
 
     MyFiles files;
-
 };
 
 
@@ -317,6 +197,9 @@ command_create_config(DBus::Connection* conn, Snapper* snapper)
     const struct option options[] = {
 	{ "fstype",		required_argument,	0,	'f' },
 	{ "template",		required_argument,	0,	't' },
+#ifdef ENABLE_ROLLBACK
+	{ "add-fstab",		no_argument,		0,	0 },
+#endif
 	{ 0, 0, 0, 0 }
     };
 
@@ -336,6 +219,7 @@ command_create_config(DBus::Connection* conn, Snapper* snapper)
 
     string fstype = "";
     string template_name = "default";
+    bool add_fstab = false;
 
     GetOpts::parsed_opts::const_iterator opt;
 
@@ -345,6 +229,9 @@ command_create_config(DBus::Connection* conn, Snapper* snapper)
     if ((opt = opts.find("template")) != opts.end())
 	template_name = opt->second;
 
+    if ((opt = opts.find("add-fstab")) != opts.end())
+	add_fstab = true;
+
     if (fstype.empty() && !Snapper::detectFstype(subvolume, fstype))
     {
 	cerr << _("Detecting filesystem type failed.") << endl;
@@ -353,7 +240,7 @@ command_create_config(DBus::Connection* conn, Snapper* snapper)
 
     if (no_dbus)
     {
-	Snapper::createConfig(config_name, subvolume, fstype, template_name);
+	Snapper::createConfig(config_name, subvolume, fstype, template_name, add_fstab);
     }
     else
     {
@@ -467,24 +354,7 @@ command_set_config(DBus::Connection* conn, Snapper* snapper)
 	exit(EXIT_FAILURE);
     }
 
-    map<string, string> raw;
-
-    while (getopts.hasArgs())
-    {
-	string arg = getopts.popArg();
-
-	string::size_type pos = arg.find("=");
-	if (pos == string::npos)
-	{
-	    cerr << _("Invalid configdata.") << endl;
-	    exit(EXIT_FAILURE);
-	}
-
-	string key = boost::trim_copy(arg.substr(0, pos));
-	string value = boost::trim_copy(arg.substr(pos + 1));
-
-	raw[key] = value;
-    }
+    map<string, string> raw = read_configdata(getopts.getArgs());
 
     if (no_dbus)
     {
@@ -1308,6 +1178,117 @@ command_undo(DBus::Connection* conn, Snapper* snapper)
 }
 
 
+#ifdef ENABLE_ROLLBACK
+
+const Filesystem*
+getFilesystem(DBus::Connection* conn, Snapper* snapper)
+{
+    XConfigInfo ci = command_get_xconfig(*conn, config_name);
+
+    map<string, string>::const_iterator it = ci.raw.find(KEY_FSTYPE);
+    if (it == ci.raw.end())
+    {
+	cerr << _("Failed to initialize filesystem handler.") << endl;
+	exit(EXIT_FAILURE);
+    }
+
+    try
+    {
+	return Filesystem::create(it->second, ci.subvolume);
+    }
+    catch (const InvalidConfigException& e)
+    {
+	cerr << _("Failed to initialize filesystem handler.") << endl;
+	exit(EXIT_FAILURE);
+    }
+}
+
+
+void
+help_rollback()
+{
+    cout << _("  Rollback:") << endl
+	 << _("\tsnapper rollback [number]") << endl
+	 << endl
+	 << _("    Options for 'rollback' command:") << endl
+	 << _("\t--print-number, -p\t\tPrint number of second created snapshot.") << endl
+	 << _("\t--description, -d <description>\tDescription for snapshots.") << endl
+	 << _("\t--cleanup-algorithm, -c <algo>\tCleanup algorithm for snapshots.") << endl
+	 << _("\t--userdata, -u <userdata>\tUserdata for snapshots.") << endl
+	 << endl;
+}
+
+
+void
+command_rollback(DBus::Connection* conn, Snapper* snapper)
+{
+    const struct option options[] = {
+	{ "print-number",       no_argument,            0,      'p' },
+	{ "description",        required_argument,      0,      'd' },
+	{ "cleanup-algorithm",  required_argument,      0,      'c' },
+	{ "userdata",           required_argument,      0,      'u' },
+	{ 0, 0, 0, 0 }
+    };
+
+    GetOpts::parsed_opts opts = getopts.parse("rollback", options);
+    if (getopts.hasArgs() && getopts.numArgs() != 1)
+    {
+	cerr << _("Command 'rollback' takes either one or no argument.") << endl;
+	exit(EXIT_FAILURE);
+    }
+
+    bool print_number = false;
+    string description;
+    string cleanup;
+    map<string, string> userdata;
+
+    GetOpts::parsed_opts::const_iterator opt;
+
+    if ((opt = opts.find("print-number")) != opts.end())
+	print_number = true;
+
+    if ((opt = opts.find("description")) != opts.end())
+	description = opt->second;
+
+    if ((opt = opts.find("cleanup-algorithm")) != opts.end())
+	cleanup = opt->second;
+
+    if ((opt = opts.find("userdata")) != opts.end())
+	userdata = read_userdata(opt->second);
+
+    const Filesystem* filesystem = getFilesystem(conn, snapper);
+    if (filesystem->fstype() != "btrfs")
+    {
+	cerr << _("Command 'rollback' only available for btrfs.") << endl;
+	exit(EXIT_FAILURE);
+    }
+
+    unsigned int tmp;
+
+    if (getopts.numArgs() == 0)
+    {
+	tmp = command_create_single_xsnapshot_of_default(*conn, config_name, true, description,
+							 cleanup, userdata);
+    }
+    else
+    {
+	tmp = read_num(getopts.popArg());
+
+	command_create_single_xsnapshot(*conn, config_name, description, cleanup, userdata);
+    }
+
+    unsigned int num2 = command_create_single_xsnapshot_v2(*conn, config_name, tmp, false,
+							   description, cleanup, userdata);
+
+    filesystem->setDefault(num2);
+
+    if (print_number)
+	cout << num2 << endl;
+}
+
+#endif
+
+
 void
 help_cleanup()
 {
@@ -1320,11 +1301,7 @@ help_cleanup()
 void
 command_cleanup(DBus::Connection* conn, Snapper* snapper)
 {
-    const struct option options[] = {
-	{ 0, 0, 0, 0 }
-    };
-
-    GetOpts::parsed_opts opts = getopts.parse("cleanup", options);
+    GetOpts::parsed_opts opts = getopts.parse("cleanup", GetOpts::no_options);
     if (getopts.numArgs() != 1)
     {
 	cerr << _("Command 'cleanup' needs one arguments.") << endl;
@@ -1519,6 +1496,9 @@ main(int argc, char** argv)
     cmds.push_back(Cmd("xadiff", command_xa_diff, help_xa_diff, false, true));
 #endif
     cmds.push_back(Cmd("undochange", command_undo, help_undo, false, true));
+#ifdef ENABLE_ROLLBACK
+    cmds.push_back(Cmd("rollback", command_rollback, help_rollback, false, true));
+#endif
     cmds.push_back(Cmd("cleanup", command_cleanup, help_cleanup, false, true));
     cmds.push_back(Cmd("debug", command_debug, help_debug, false, false));
 
@@ -1549,10 +1529,10 @@ main(int argc, char** argv)
     {
 	unsigned int s;
 	opt->second >> s;
-	if (s >= _End)
+	if (s >= Table::numStyles)
 	{
 	    cerr << sformat(_("Invalid table style %d."), s) << " "
-		 << sformat(_("Use an integer number from %d to %d"), 0, _End - 1) << endl;
+		 << sformat(_("Use an integer number from %d to %d."), 0, Table::numStyles - 1) << endl;
 	    exit(EXIT_FAILURE);
 	}
 	Table::defaultStyle = (TableLineStyle) s;
@@ -1668,43 +1648,7 @@ main(int argc, char** argv)
 	}
 	catch (const DBus::ErrorException& e)
 	{
-	    string name = e.name();
-	    if (name == "error.unknown_config")
-		cerr << _("Unknown config.") << endl;
-	    else if (name == "error.no_permissions")
-		cerr << _("No permissions.") << endl;
-	    else if (name == "error.invalid_userdata")
-		cerr << _("Invalid userdata.") << endl;
-	    else if (name == "error.invalid_configdata")
-		cerr << _("Invalid configdata.") << endl;
-	    else if (name == "error.illegal_snapshot")
-		cerr << _("Illegal Snapshot.") << endl;
-	    else if (name == "error.config_locked")
-		cerr << _("Config is locked.") << endl;
-	    else if (name == "error.config_in_use")
-		cerr << _("Config is in use.") << endl;
-	    else if (name == "error.snapshot_in_use")
-		cerr << _("Snapshot is in use.") << endl;
-	    else if (name == "error.unknown_file_use")
-		cerr << _("Unknown file.") << endl;
-	    else if (name == "error.io_error")
-		cerr << _("IO Error.") << endl;
-	    else if (name == "error.create_config_failed")
-		cerr << sformat(_("Creating config failed (%s)."), e.message()) << endl;
-	    else if (name == "error.delete_config_failed")
-		cerr << sformat(_("Deleting config failed (%s)."), e.message()) << endl;
-	    else if (name == "error.create_snapshot_failed")
-		cerr << _("Creating snapshot failed.") << endl;
-	    else if (name == "error.delete_snapshot_failed")
-		cerr << _("Deleting snapshot failed.") << endl;
-	    else if (name == "error.invalid_user")
-		cerr << _("Invalid user.") << endl;
-	    else if (name == "error.invalid_group")
-		cerr << _("Invalid group.") << endl;
-	    else if (name == "error.acl_error")
-		cerr << _("ACL error.") << endl;
-	    else
-		cerr << _("Failure") << " (" << name << ")." << endl;
+	    cerr << error_description(e) << endl;
 	    exit(EXIT_FAILURE);
 	}
 	catch (const DBus::FatalException& e)
