@@ -1,5 +1,5 @@
 /*
- * Copyright (c) [2011-2013] Novell, Inc.
+ * Copyright (c) [2011-2014] Novell, Inc.
  *
  * All Rights Reserved.
  *
@@ -30,6 +30,9 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <asm/types.h>
+#ifdef ENABLE_ROLLBACK
+#include <libmount/libmount.h>
+#endif
 #ifdef HAVE_LIBBTRFS
 #include <btrfs/ioctl.h>
 #include <btrfs/send.h>
@@ -47,10 +50,13 @@
 #include "snapper/Snapper.h"
 #include "snapper/SnapperTmpl.h"
 #include "snapper/SnapperDefines.h"
+#include "snapper/Acls.h"
 
 
 namespace snapper
 {
+    using namespace std;
+
 
     Filesystem*
     Btrfs::create(const string& fstype, const string& subvolume)
@@ -69,13 +75,17 @@ namespace snapper
 
 
     void
-    Btrfs::createConfig() const
+    Btrfs::createConfig(bool add_fstab) const
     {
 	SDir subvolume_dir = openSubvolumeDir();
 
-	if (!create_subvolume(subvolume_dir.fd(), ".snapshots"))
+	try
 	{
-	    y2err("create subvolume failed errno:" << errno << " (" << stringerror(errno) << ")");
+	    create_subvolume(subvolume_dir.fd(), ".snapshots");
+	}
+	catch (const runtime_error& e)
+	{
+	    y2err("create subvolume failed, " << e.what());
 	    throw CreateConfigFailedException("creating btrfs snapshot failed");
 	}
 
@@ -83,6 +93,20 @@ namespace snapper
 	struct stat stat;
 	if (x.stat(&stat, 0) == 0)
 	    x.chmod(stat.st_mode & ~0027, 0);
+
+#ifdef ENABLE_ROLLBACK
+	if (subvolume == "/" && add_fstab)
+	{
+	    try
+	    {
+		addToFstab();
+	    }
+	    catch (const runtime_error& e)
+	    {
+		y2err("adding to fstab failed, " << e.what());
+	    }
+	}
+#endif
     }
 
 
@@ -91,9 +115,29 @@ namespace snapper
     {
 	SDir subvolume_dir = openSubvolumeDir();
 
-	if (!delete_subvolume(subvolume_dir.fd(), ".snapshots"))
+#ifdef ENABLE_ROLLBACK
+	if (subvolume == "/")
 	{
-	    y2err("delete subvolume failed errno:" << errno << " (" << stringerror(errno) << ")");
+	    subvolume_dir.umount(".snapshots");
+
+	    try
+	    {
+		removeFromFstab();
+	    }
+	    catch (const runtime_error& e)
+	    {
+		y2err("removing from fstab failed, " << e.what());
+	    }
+	}
+#endif
+
+	try
+	{
+	    delete_subvolume(subvolume_dir.fd(), ".snapshots");
+	}
+	catch (const runtime_error& e)
+	{
+	    y2err("delete subvolume failed, " << e.what());
 	    throw DeleteConfigFailedException("deleting btrfs snapshot failed");
 	}
     }
@@ -179,27 +223,99 @@ namespace snapper
 
 
     void
-    Btrfs::createSnapshot(unsigned int num) const
+    Btrfs::createSnapshot(unsigned int num, unsigned int num_parent, bool read_only) const
+    {
+	if (num_parent == 0)
+	{
+	    SDir subvolume_dir = openSubvolumeDir();
+	    SDir info_dir = openInfoDir(num);
+
+	    try
+	    {
+		create_snapshot(subvolume_dir.fd(), info_dir.fd(), "snapshot", read_only);
+	    }
+	    catch (const runtime_error& e)
+	    {
+		y2err("create snapshot failed, " << e.what());
+		throw CreateSnapshotFailedException();
+	    }
+	}
+	else
+	{
+	    SDir snapshot_dir = openSnapshotDir(num_parent);
+	    SDir info_dir = openInfoDir(num);
+
+	    try
+	    {
+		create_snapshot(snapshot_dir.fd(), info_dir.fd(), "snapshot", read_only);
+	    }
+	    catch (const runtime_error& e)
+	    {
+		y2err("create snapshot failed, " << e.what());
+		throw CreateSnapshotFailedException();
+	    }
+	}
+    }
+
+
+#ifdef ENABLE_ROLLBACK
+
+    void
+    Btrfs::createSnapshotOfDefault(unsigned int num, bool read_only) const
     {
 	SDir subvolume_dir = openSubvolumeDir();
+	unsigned long long id = get_default_id(subvolume_dir.fd());
+	string name = get_subvolume(subvolume_dir.fd(), id);
+
+	bool found = false;
+	MtabData mtab_data;
+	if (!getMtabData(subvolume, found, mtab_data))
+	{
+	    y2err("failed to find device");
+	    throw CreateSnapshotFailedException();
+	}
+
+	SDir infos_dir = openInfosDir();
+	TmpMount tmp_mount(infos_dir, mtab_data.device, "tmp-mnt-XXXXXX", "btrfs", 0,
+			   "subvol=" + name);
+
+	SDir tmp_mount_dir(infos_dir, tmp_mount.getName());
 	SDir info_dir = openInfoDir(num);
 
-	if (!create_snapshot(subvolume_dir.fd(), info_dir.fd(), "snapshot"))
+	try
 	{
-	    y2err("create snapshot failed errno:" << errno << " (" << stringerror(errno) << ")");
+	    create_snapshot(tmp_mount_dir.fd(), info_dir.fd(), "snapshot", read_only);
+	}
+	catch (const runtime_error& e)
+	{
+	    y2err("create snapshot failed, " << e.what());
 	    throw CreateSnapshotFailedException();
 	}
     }
 
+#else
+
+    void
+    Btrfs::createSnapshotOfDefault(unsigned int num, bool read_only) const
+    {
+	throw std::logic_error("not implemented");
+    }
+
+#endif
+
 
     void
     Btrfs::deleteSnapshot(unsigned int num) const
     {
 	SDir info_dir = openInfoDir(num);
 
-	if (!delete_subvolume(info_dir.fd(), "snapshot"))
+	try
 	{
-	    y2err("delete snapshot failed errno:" << errno << " (" << stringerror(errno) << ")");
+	    delete_subvolume(info_dir.fd(), "snapshot");
+	}
+	catch (const runtime_error& e)
+	{
+	    y2err("delete snapshot failed, " << e.what());
 	    throw DeleteSnapshotFailedException();
 	}
     }
@@ -224,6 +340,14 @@ namespace snapper
     }
 
 
+    bool
+    Btrfs::isSnapshotReadOnly(unsigned int num) const
+    {
+	SDir snapshot_dir = openSnapshotDir(num);
+	return is_subvolume_read_only(snapshot_dir.fd());
+    }
+
+
     bool
     Btrfs::checkSnapshot(unsigned int num) const
     {
@@ -450,10 +574,10 @@ namespace snapper
 	if (status & CREATED) status = CREATED;
 	if (status & DELETED) status = DELETED;
 
-	if (status & (CONTENT | PERMISSIONS | USER | GROUP | XATTRS))
+	if (status & (CONTENT | PERMISSIONS | OWNER | GROUP | XATTRS | ACL))
 	{
 	    // TODO check for content sometimes not required
-	    status &= ~(CONTENT | PERMISSIONS | USER | GROUP | XATTRS);
+	    status &= ~(CONTENT | PERMISSIONS | OWNER | GROUP | XATTRS | ACL);
 
 	    string dirname = snapper::dirname(name);
 	    string basename = snapper::basename(name);
@@ -546,7 +670,7 @@ namespace snapper
 	else
 	{
 	    node->status &= ~(CREATED | DELETED);
-	    node->status |= CONTENT | PERMISSIONS | USER | GROUP | XATTRS;
+	    node->status |= CONTENT | PERMISSIONS | OWNER | GROUP | XATTRS | ACL;
 	}
     }
 
@@ -679,7 +803,7 @@ namespace snapper
 		else
 		{
 		    node->status &= ~(CREATED | DELETED);
-		    node->status |= CONTENT | PERMISSIONS | USER | GROUP | XATTRS;
+		    node->status |= CONTENT | PERMISSIONS | OWNER | GROUP | XATTRS | ACL;
 		}
 
 		merge(processor, &it->second, from, to, x);
@@ -697,7 +821,7 @@ namespace snapper
 		else
 		{
 		    node->status &= ~(CREATED | DELETED);
-		    node->status |= CONTENT | PERMISSIONS | USER | GROUP | XATTRS;
+		    node->status |= CONTENT | PERMISSIONS | OWNER | GROUP | XATTRS | ACL;
 		}
 
 		merge(processor, &it->second, from, to, x);
@@ -854,6 +978,14 @@ namespace snapper
 
 	tree_node* node = processor->files.insert(path);
 	node->status |= XATTRS;
+
+	if (is_acl_signature(name))
+	{
+	    #ifdef DEBUG_PROCESS
+		y2deb("adding acl flag, signature:'" << name << "'");
+	    #endif
+	    node->status |= ACL;
+	}
 #endif
 
 	return 0;
@@ -872,6 +1004,14 @@ namespace snapper
 
 	tree_node* node = processor->files.insert(path);
 	node->status |= XATTRS;
+
+	if (is_acl_signature(name))
+	{
+	    #ifdef DEBUG_PROCESS
+		y2deb("adding acl flag, signature:'" << name << "'");
+	    #endif
+	    node->status |= ACL;
+	}
 #endif
 
 	return 0;
@@ -920,7 +1060,7 @@ namespace snapper
 #endif
 
 	tree_node* node = processor->files.insert(path);
-	node->status |= USER | GROUP;
+	node->status |= OWNER | GROUP;
 
 	return 0;
     }
@@ -1159,4 +1299,150 @@ namespace snapper
 #endif
 
 
+
+#ifdef ENABLE_ROLLBACK
+
+    void
+    Btrfs::setDefault(unsigned int num) const
+    {
+	try
+	{
+	    if (num == 0)
+	    {
+		SDir subvolume_dir = openSubvolumeDir();
+		unsigned long long id = get_id(subvolume_dir.fd());
+		set_default_id(subvolume_dir.fd(), id);
+	    }
+	    else
+	    {
+		SDir snapshot_dir = openSnapshotDir(num);
+		unsigned long long id = get_id(snapshot_dir.fd());
+
+		SDir subvolume_dir = openSubvolumeDir();
+		set_default_id(subvolume_dir.fd(), id);
+	    }
+	}
+	catch (const runtime_error& e)
+	{
+	    y2err("set default failed, " << e.what());
+	    throw IOErrorException();
+	}
+    }
+
+#else
+
+    void
+    Btrfs::setDefault(unsigned int num) const
+    {
+	throw std::logic_error("not implemented");
+    }
+
+#endif
+
+
+#ifdef ENABLE_ROLLBACK
+
+    class MntTable
+    {
+
+    public:
+
+	MntTable()
+	    : table(mnt_new_table())
+	{
+	    if (!table)
+		throw runtime_error("mnt_new_table failed");
+
+	    mnt_table_enable_comments(table, 1);
+	}
+
+	~MntTable()
+	{
+	    mnt_reset_table(table);
+	}
+
+	void parse_fstab()
+	{
+	    if (mnt_table_parse_fstab(table, "/etc/fstab") != 0)
+		throw runtime_error("mnt_table_parse_fstab failed");
+	}
+
+	void replace_file()
+	{
+	    if (mnt_table_replace_file(table, "/etc/fstab") != 0)
+		throw runtime_error("mnt_table_replace_file failed");
+	}
+
+	struct libmnt_fs* find_target(const string& path, int directon)
+	{
+	    return mnt_table_find_target(table, path.c_str(), directon);
+	}
+
+	void add_fs(struct libmnt_fs* fs)
+	{
+	    if (mnt_table_add_fs(table, fs) != 0)
+		throw runtime_error("mnt_table_add_fs failed");
+	}
+
+	void remove_fs(struct libmnt_fs* fs)
+	{
+	    if (mnt_table_remove_fs(table, fs) != 0)
+		throw runtime_error("mnt_table_remove_fs failed");
+	}
+
+    private:
+
+	struct libmnt_table* table;
+
+    };
+
+
+    void
+    Btrfs::addToFstab() const
+    {
+	SDir infos_dir = openInfosDir();
+	unsigned long long id = get_id(infos_dir.fd());
+	string subvol_option = get_subvolume(infos_dir.fd(), id);
+
+	MntTable mnt_table;
+	mnt_table.parse_fstab();
+
+	libmnt_fs* root = mnt_table.find_target(subvolume, MNT_ITER_FORWARD);
+	if (!root)
+	    throw runtime_error("root entry not found");
+
+	libmnt_fs* snapshots = mnt_copy_fs(NULL, root);
+	if (!snapshots)
+	    throw runtime_error("mnt_copy_fs failed");
+
+	string mountpoint = (subvolume == "/" ? "" : subvolume) +  "/.snapshots";
+	mnt_fs_set_target(snapshots, mountpoint.c_str());
+
+	char* options = mnt_fs_strdup_options(snapshots);
+	mnt_optstr_remove_option(&options, "defaults");
+	mnt_optstr_set_option(&options, "subvol", subvol_option.c_str());
+	mnt_fs_set_options(snapshots, options);
+	free(options);
+
+	mnt_table.add_fs(snapshots);
+	mnt_table.replace_file();
+    }
+
+
+    void
+    Btrfs::removeFromFstab() const
+    {
+	MntTable mnt_table;
+	mnt_table.parse_fstab();
+
+	string mountpoint = (subvolume == "/" ? "" : subvolume) +  "/.snapshots";
+	libmnt_fs* snapshots = mnt_table.find_target(mountpoint, MNT_ITER_FORWARD);
+	if (!snapshots)
+	    return;
+
+	mnt_table.remove_fs(snapshots);
+	mnt_table.replace_file();
+    }
+
+#endif
 }
