@@ -1,5 +1,5 @@
 /*
- * Copyright (c) [2011-2013] Novell, Inc.
+ * Copyright (c) [2011-2014] Novell, Inc.
  *
  * All Rights Reserved.
  *
@@ -24,14 +24,27 @@
 #define SNAPPER_BTRFS_UTILS_H
 
 
+#include <string>
+
+
 namespace snapper
 {
+    using std::string;
+
 
     bool is_subvolume(const struct stat& stat);
 
-    bool create_subvolume(int fddst, const string& name);
-    bool create_snapshot(int fd, int fddst, const string& name);
-    bool delete_subvolume(int fd, const string& name);
+    bool is_subvolume_read_only(int fd);
+
+    void create_subvolume(int fddst, const string& name);
+    void create_snapshot(int fd, int fddst, const string& name, bool read_only);
+    void delete_subvolume(int fd, const string& name);
+
+    void set_default_id(int fd, unsigned long long id);
+    unsigned long long get_default_id(int fd);
+
+    string get_subvolume(int fd, unsigned long long id);
+    unsigned long long get_id(int fd);
 
 }
 
