@@ -95,6 +95,10 @@ namespace snapper
 	config_info->getValue(KEY_FSTYPE, fstype);
 	filesystem = Filesystem::create(fstype, config_info->getSubvolume());
 
+	bool sync_acl;
+	if (config_info->getValue(KEY_SYNC_ACL, sync_acl) && sync_acl == true)
+	    syncAcl();
+
 	y2mil("subvolume:" << config_info->getSubvolume() << " filesystem:" <<
 	      filesystem->fstype());
 
@@ -209,6 +213,28 @@ namespace snapper
     }
 
 
+    Snapshots::iterator
+    Snapper::createSingleSnapshot(Snapshots::const_iterator parent, bool read_only, uid_t uid,
+				  const string& description, const string& cleanup,
+				  const map<string, string>& userdata)
+    {
+	if (parent == snapshots.end())
+	    throw IllegalSnapshotException();
+
+	return snapshots.createSingleSnapshot(parent, read_only, uid, description, cleanup,
+					      userdata);
+    }
+
+
+    Snapshots::iterator
+    Snapper::createSingleSnapshotOfDefault(bool read_only, uid_t uid, const string& description,
+					   const string& cleanup,
+					   const map<string, string>& userdata)
+    {
+	return snapshots.createSingleSnapshotOfDefault(read_only, uid, description, cleanup,
+						       userdata);
+    }
+
 
     Snapshots::iterator
     Snapper::createPreSnapshot(uid_t uid, const string& description, const string& cleanup,
@@ -290,6 +316,15 @@ namespace snapper
     void
     Snapper::createConfig(const string& config_name, const string& subvolume,
 			  const string& fstype, const string& template_name)
+    {
+	createConfig(config_name, subvolume, fstype, template_name, false);
+    }
+
+
+    void
+    Snapper::createConfig(const string& config_name, const string& subvolume,
+			  const string& fstype, const string& template_name,
+			  bool add_fstab)
     {
 	y2mil("Snapper create-config");
 	y2mil("libsnapper version " VERSION);
@@ -370,9 +405,9 @@ namespace snapper
 	    throw CreateConfigFailedException("modifying config failed");
 	}
 
-	filesystem->createConfig();
+	filesystem->createConfig(add_fstab);
 
-#if 1
+#ifdef ENABLE_ROLLBACK
 	if (subvolume == "/" && filesystem->fstype() == "btrfs" &&
 	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
 	{
@@ -390,7 +425,7 @@ namespace snapper
 
 	auto_ptr<Snapper> snapper(new Snapper(config_name));
 
-#if 1
+#ifdef ENABLE_ROLLBACK
 	if (snapper->subvolumeDir() == "/" && snapper->getFilesystem()->fstype() == "btrfs" &&
 	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
 	{
@@ -533,7 +568,11 @@ namespace snapper
     {
 	SDir infos_dir = openInfosDir();
 
-	acl_t acl = acl_get_fd(infos_dir.fd());
+	acl_t orig_acl = acl_get_fd(infos_dir.fd());
+	if (!orig_acl)
+	    throw AclException();
+
+	acl_t acl = acl_dup(orig_acl);
 	if (!acl)
 	    throw AclException();
 
@@ -609,8 +648,9 @@ namespace snapper
 	if (acl_calc_mask(&acl) != 0)
 	    throw AclException();
 
-	if (acl_set_fd(infos_dir.fd(), acl) != 0)
-	    throw AclException();
+	if (acl_cmp(orig_acl, acl) == 1)
+	    if (acl_set_fd(infos_dir.fd(), acl) != 0)
+		throw AclException();
 
 	if (acl_free(acl) != 0)
 	    throw AclException();
